///Copyright 2018 Miolin

///Licensed under the Apache License, Version 2.0 (the "License");
///you may not use this file except in compliance with the License.
///You may obtain a copy of the License at

///http://www.apache.org/licenses/LICENSE-2.0

///Unless required by applicable law or agreed to in writing, software
///distributed under the License is distributed on an "AS IS" BASIS,
///WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
///See the License for the specific language governing permissions and
///limitations under the License.


library state_controller;

import 'package:flutter/widgets.dart';


mixin ControllerInjector<W extends StatefulWidget, SC extends StateController<W>> on State<W> {
  SC _controller;
  SC get controller => _controller;

  @protected
  @mustCallSuper
  void initState() {
    _controller = createStateController();
    _controller._state = this;
    _controller._context = context;
    _controller._widget = widget;
    _controller.onCreate();
    super.initState();
  }

  SC createStateController();

  @protected
  @mustCallSuper
  void didUpdateWidget(covariant W oldWidget) {
    _controller._context = context;
    _controller._widget = widget;
    _controller.onWidgetUpdate();
    super.didUpdateWidget(oldWidget);
  }

  @protected
  @mustCallSuper
  void dispose() {
    _controller.onDispose();
    _controller = null;
    super.dispose();
  }
}

abstract class StateController<T extends StatefulWidget> {
  State _state;
  BuildContext _context;
  T _widget;
  T get widget => _widget;
  BuildContext get context => _context;

  @protected
  void onCreate() {}

  @protected
  void onWidgetUpdate() {}

  void changeState(VoidCallback fn) {
    if (_state == null) return;
    // ignore: invalid_use_of_protected_member
    _state.setState(fn);
  }

  @protected
  @mustCallSuper
  void onDispose() {
    _context = null;
    _widget = null;
    _state = null;
  }
}
